module StudioGame
	class Game
		require_relative 'player'
		require_relative 'die'
		require_relative 'game_turn'
		require 'csv'

		attr_reader :title

		def initialize(title)
			@title = title
			@players = []
		end

		def high_score_entry(player)
			formatted_name = player.name.ljust(20, '.')
			"#{formatted_name} #{player.score}"
		end

		def save_high_scores(to_file = "high_scores.txt")
			File.open(to_file, "w") do |file|
				file.puts "#{@title} High Scores:"
				@players.sort.each do |player|
					file.puts high_score_entry(player)
				end
			end
		end


		def load_players(players_file)
			CSV.foreach(players_file) do |row|
				player = Player.new(row[0], row[1].to_i)
				add_player(player)
			end
		end

		def add_player(player)
			@players << player
		end

		def play(rounds)
			puts "There are #{@players.size} players in #{@title}: "

			@players.each do |player|
				puts player
			end

			1.upto(rounds) do |round|
				if block_given?
					break if yield
				end			
				puts "\nRound #{round}:"
				@players.each do |player|
					GameTurn.take_turn(player)
					puts player
				end
			end
		end

		def print_name_and_health(player) 
			puts "#{player.name} (#{player.health})"
		end

		def total_points
			@players.reduce(0) { |sum, player| sum + player.points }
		end	

		def print_stats
			strong_players, wimpy_players = @players.partition {|player| player.strong?}
			puts "\n#{@title} Statistics:"

			puts "\n#{strong_players.size} strong players:"
			strong_players.each do |player|
				print_name_and_health(player)
			end


			puts "\n#{wimpy_players.size} wimpy players:"
			wimpy_players.each do |player|
				print_name_and_health(player)
			end	

			puts "\n#{@title} High Scores:"	
			sorted_players = strong_players.sort 
			sorted_players.each do |player|
				puts high_score_entry(player)
			end

			@players.sort.each do |player|
				puts "\n#{player.name}'s point totals:"
				player.each_found_treasure do |treasure|
					puts "#{treasure.points} total #{treasure.name} points"
				end
				puts "#{player.points} grand total points"
			end

			puts "\n\n#{total_points} total points from treasures found"
		end
	end
end

if __FILE__ == $0	
    puts "====================================" "\nSAMPLE GAME CODE:\n" 
	player = Player.new("moe")
	puts player.name
	puts player.health
	player.w00t
	player.blam
	puts player.health
end